import es, gamethread, os.path, popuplib, playerlib, cPickle, services, re

try:
	import nativetools
except ImportError:
	nativetools = None
	
info = es.AddonInfo() 
info['name']        = "bmute" 
info['version']     = "2.0.0"
info['author']      = "Bonbon AKA: Bonbon367" 
info['url']         = "http://addons.eventscripts.com/addons/view/bmute" 
info['description'] = "Mute control for your server"

sv_options = {
	'bmute_enable_votemute': 1,
	'bmute_vote_ratio': 0.5,
	'bmute_min_votes': 3,
	'bmute_votemute_time': 6000,
	'bmute_admins': 'STEAM_0:0:11089864,STEAM_0:0:0000000,mani_admins',
	'bmute_menu_mute_reasons': 'I don\'t like you|Spamming',
	'bmute_forum_link': 'None',
}

sv = es.ServerVar
sv('bmute_version', info['version'], info['description']).makepublic()

FILEPATH = es.getAddonPath('bmute') + '/'

### Loads ###

def load():
	global HASEST, HASMANI, HASSM
	HASMANI = not str(es.ServerVar('mani_admin_plugin_version')) == '0'
	HASSM = not str(es.ServerVar('sourcemod_version')) == '0'

	if not es.exists('saycommand', '!votemute'):
		es.regsaycmd('!votemute', 'bmute/votemute_command')

	for option in sv_options:
		sv(option).set(sv_options[option])

	if os.path.isfile(FILEPATH + 'bmute.cfg'):
		es.server.cmd('es_xmexec ../addons/eventscripts/bmute/bmute.cfg')

	if not es.exists('saycommand', '!mute'):
		es.regsaycmd('!mute', 'bmute/mute_command', 'Mute a player')

	if not es.exists('saycommand', '!unmute'):
		es.regsaycmd('!unmute', 'bmute/unmute', 'Unmute a player')

	if not es.exists('clientcommand', 'mute_status'):
		es.regclientcmd('mute_status', 'bmute/status', 'Check your muted status')

	if not es.exists('saycommand', '!mutemenu'):
		es.regsaycmd('!mutemenu', 'bmute/mutemenu')

	if not es.exists('saycommand', '!muted'):
		es.regsaycmd('!muted', 'bmute/show_muted')

	mute.unmuteLoop()
	mute.setupAuth()

def unload():
	if es.exists('saycommand', '!votemute'):
		es.unregsaycmd('!votemute')

	if es.exists('saycommand', '!mute'):
		es.unregsaycmd('!mute')

	if es.exists('saycommand', '!unmute'):
		es.unregsaycmd('!unmute')

	if es.exists('clientcommand', 'mute_status'):
		es.unregclientcmd('mute_status')

	if es.exists('saycommand', '!muted'):
		es.unregsaycmd('!muted')

	for delay in ['update_mute', 'lower times', 'votemute prune']:
		gamethread.cancelDelayed(delay)

	for steamid in vote.players:
		gamethread.cancelDelayed('votemute remove %s'%steamid)

### Classes ###

class MuteManager(object):
	def __init__(self):
		self.muted = self.getMuted()

	### Data ###

	@staticmethod
	def getMuted():
		muted = {}
		oFilePath = FILEPATH + 'muted.db'

		if os.path.isfile(oFilePath):
			oFile = open(oFilePath)
			muted = cPickle.load(oFile)
			oFile.close()
		return muted

	def saveData(self):
		iFile = open(FILEPATH + 'muted.db', 'w')
		cPickle.dump(self.muted, iFile)
		iFile.close()

	### Auth ###

	def isAuthed(self, userid):
		if self.hasAuth(userid) or es.getplayersteamid(userid) in self.admins:
			return True

	def setupAuth(self):
		if services.isRegistered('auth'):
			auth_service = services.use('auth')
			auth_service.registerCapability('bmute_admin', auth_service.ADMIN)
			self.hasAuth = lambda x: auth_service.isUseridAuthorized(x, 'bmute_admin')

		else:
			self.hasAuth = lambda x: False
		self.getAdmins()

	def getAdmins(self):
		self.admins = str(sv('bmute_admins')).split(',')
		if 'mani_admins' in self.admins:
			self.admins += self.getManiAdmins()

	@staticmethod
	def getManiAdmins():
		clientsPath = FILEPATH.replace('addons/eventscripts/bmute/', 'cfg/mani_admin_plugin/clients.txt')
		if not os.path.isfile(clientsPath):
			return []

		oFile = open(clientsPath)
		admins = map(lambda x: x.upper(), re.findall('(STEAM_0:[0-1]:\d+)', oFile.read(), re.IGNORECASE))
		oFile.close()
 
		return admins

	### Misc ###

	def addPlayer(self, steamid, mtime, reason, muter='Server'):
		self.muted[steamid] = {'reason': reason, 'time': mtime, 'muter': muter, 'name': 'None'}
		userid = es.getuserid(steamid)

		if userid:
			self.muted[steamid]['name'] = es.getplayername(userid)
			es.cexec(userid, 'mute_status')
			self.mutePlayer(userid)
		self.saveData()

	def mutePlayer(self, userid, mode=True):
		if HASSM:
			if mode:
				es.server.queuecmd('sm_mute #%s'%userid)
			else:
				es.server.queuecmd('sm_unmute #%s'%userid)

		elif nativetools:
			nativetools.mutePlayer(userid, mode)

		elif HASMANI:
			es.server.queuecmd('ma_mute %s %s'%(userid, 1 if mode else 0))

	def unmuteLoop(self):
		gamethread.delayedname(1, 'update_mute', self.unmuteLoop)
		for steamid in self.muted.keys():
			self.muted[steamid]['time'] -= 1
			if self.muted[steamid]['time'] <= 0:
				userid = es.getuserid(steamid)
				if userid:
					self.mutePlayer(userid, False)
					es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou have been unmuted, better behave!')

				del self.muted[steamid]
				self.saveData()

mute = MuteManager()

class VoteManager(object):
	def __init__(self):
		self.players = {}

	@staticmethod
	def getNeeded():
		return max(int(sv('bmute_min_votes')), int(es.getplayercount() * float(sv('bmute_vote_ratio'))))

	### Player Management ###

	def playerRejoin(self, steamid):
		self.players[steamid]['against'] = 0
		for player in self.players:
			if steamid in self.players[player]['for']:
				self.players[steamid]['against'] += 1

	def removePlayer(self, steamid):
		for player in self.players:
			if steamid in self.players[player]['for']:
				self.players[player]['for'].remove(steamid)
		del self.players[steamid]

	def prunePlayers(self):
		steamids = self.players.keys()
		for steamid in self.players:
			for for_steamid in list(self.players[steamid]['for']):
				if not for_steamid in steamids:
					self.players[steamid]['for'].remove(for_steamid)

	### Popups ###
	def sendPopup(self, userid):
		selectPlayerMenu = popuplib.easymenu('Select VTM Player', '_popup_choice', self.selectPlayerMenuSelect)
		for tuserid in es.getUseridList():
			if tuserid == userid:
				continue

			tsteamid = es.getplayersteamid(tuserid)
			if tsteamid in mute.muted:
				continue
			selectPlayerMenu.addoption(tuserid, '%s [%s]'%(es.getplayername(tuserid), self.players.get(tsteamid, {}).get('against', 0)))
		popuplib.send('Select VTM Player', userid)

	def selectPlayerMenuSelect(self, userid, choice, popupid):
		steamid = es.getplayersteamid(userid)
		csteamid = es.getplayersteamid(choice)

		if not es.exists('userid', choice):
			es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultPlayer has left the server!')
			return

		if not steamid in self.players:
			self.players[steamid] = {'for': [], 'against': 0}

		if csteamid in self.players[steamid]['for']:
			es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou have already voted against #lightgreen%s'%es.getplayername(choice))
			return

		self.players[steamid]['for'].append(csteamid)
		if not csteamid in self.players:
			self.players[csteamid] = {'for': [], 'against': 0}

		self.players[csteamid]['against'] += 1
		es.msg('#multi', '#lightgreen[Mute]: %s #defaultvoted to mute #lightgreen%s #green[%s/%s] needed'%(es.getplayername(userid), es.getplayername(choice), self.players[csteamid]['against'], self.getNeeded()))

		if self.players[csteamid]['against'] >= self.getNeeded():
			self.players[csteamid]['against'] = 0
			es.msg('#multi', '#lightgreen[Mute]: %s#default has been vote muted!'%es.getplayername(choice))
			mute.addPlayer(csteamid, int(sv('bmute_votemute_time')), 'Votemute')

vote = VoteManager()

### Events ###

def player_activate(ev):
	steamid = ev['es_steamid']
	if steamid in vote.players:
		vote.playerRejoin(steamid)
		gamethread.cancelDelayed('votemute remove %s'%steamid)

	if steamid in mute.muted:
		mute.mutePlayer(ev['userid'])

def player_spawn(ev):
	if ev['es_steamid'] in mute.muted:
		es.tell(ev['userid'], '#multi', '#lightgreen[Mute]: #defaultYou are muted, type #lightgreenmute_status#default to see the details!')

def player_disconnect(ev):
	steamid = ev['es_steamid']
	if steamid in vote.players:
		for for_steamid in vote.players[steamid]['for']:
			vote.players[for_steamid]['against'] -= 1
		del vote.players[steamid]['for'][:]

		if not vote.players[steamid]['against']:
			del vote.players[steamid]
			return
		gamethread.delayedname(300, 'votemute remove %s'%steamid, vote.removePlayer, steamid)

def round_start(ev):
	gamethread.delayedname(5, 'votemute prune', vote.prunePlayers)

### Commands ###

def mute_command():
	args = es.getargs()
	userid = es.getcmduserid()

	if not mute.isAuthed(userid):
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou are not authorized to run this command!')
		return

	if not args:
		args = ''

	args = args.split(' ')
	if len(args) < 2:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultInvalid syntax --#lightgreen !mute <userid/name/steamid> <time> [reason]')
		return

	target = es.getuserid(args[0].replace(' ', ''))
	if re.match('(STEAM_0:[0-1]:\d+)', args[0].replace(' ', '')):
		target = args[0].replace(' ', '')

	if not target:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultPlayer does not exist!')
		return

	if type(target) is int:
		target = es.getplayersteamid(target)

	mtime = args[1]
	try:
		mtime = int(mtime)

	except ValueError:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultTime argument must be an integer!')
		return

	reason = ' '.join(args[2:])
	if not reason:
		reason = 'None'
	mute.addPlayer(target, mtime, reason, es.getplayername(userid))

def unmute():
	args = es.getargs()
	userid = es.getcmduserid()

	if not mute.isAuthed(userid):
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou are not authorized to run this command!')
		return

	if not args:
		args = ''

	args = args.split(' ')
	if not args:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultInvalid syntax --#lightgreen !unmute <userid/name/steamid>')
		return

	target = es.getuserid(args[0].replace(' ', ''))
	if re.match('(STEAM_0:[0-1]:\d+)', args[0].replace(' ', '')):
		target = args[0].replace(' ', '')

	if not target:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultPlayer does not exist!')
		return

	if type(target) is int:
		target = es.getplayersteamid(target)

	if not target in mute.muted:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultSteamid #lightgreen%s#default is not muted!'%target)
		return

	mute.muted[target]['time'] = 2
	es.tell(userid, '#multi', '#lightgreen[Mute]: %s#default has been unmuted!'%mute.muted[target]['name'])

def status():
	userid = es.getcmduserid()
	steamid = es.getplayersteamid(userid)

	if not steamid in mute.muted:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou are not muted!')
		return

	menu = popuplib.easymenu('You Are Muted', '_popup_choice', muted_menuselect)
	muteData = mute.muted[steamid]

	minutes, seconds = divmod(muteData['time'], 60)
	hours, minutes = divmod(minutes, 60)

	menu.addoption('time', 'Time: %s:%02i:%02i'%(hours, minutes, seconds))
	menu.addoption('reason', 'Reason: %s'%muteData['reason'])
	menu.addoption('muter', 'Muter: %s'%muteData['muter'])

	if not str(sv('bmute_forum_link')) == 'None':
		menu.addoption('link', 'Complain: %s'%str(sv('bmute_forum_link')))
	popuplib.send('You Are Muted', userid)

def mutemenu():
	userid = es.getcmduserid()
	if not mute.isAuthed(userid):
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou are not authorized to run this command!')
		return

	muteMenu = popuplib.easymenu('Mute Menu', '_popup_choice', muteMenuSelect)
	for userid2 in es.getUseridList():
		muteMenu.addoption(es.getplayersteamid(userid2), es.getplayername(userid2))
	popuplib.send('Mute Menu', userid)

def show_muted():
	userid = es.getcmduserid()
	if not mute.isAuthed(userid):
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultYou are not authorized to run this command!')
		return

	mutedPlayersMenu = popuplib.easymenu('Muted Players', '_popup_choice', mutedPlayersMenuSelect)
	for steamid in mute.muted:
		mutedPlayersMenu.addoption(steamid, mute.muted[steamid]['name'])
	popuplib.send('Muted Players', userid)

def votemute_command():
	vote.sendPopup(es.getcmduserid())

### Popups ###

def muted_menuselect(userid, choice, popupid):
	es.cexec(userid, 'mute_status')

def mutedPlayersMenuSelect(userid, choice, popupid):
	if not choice in mute.muted:
		es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultPlayer is no longer muted!')
		return

	playerMuteInfoMenu = pouplib.easymenu('Muted Player Info', '_popup_choice', playerMuteInfoMenuSelect)
	muteData = mute.muted[choice]

	minutes, seconds = divmod(muteData['time'], 60)
	hours, minutes = divmod(minutes, 60)

	playerMuteInfoMenu.addoption('time', 'Time: %s:%02i:%02i'%(hours, minutes, seconds))
	playerMuteInfoMenu.addoption('reason', 'Reason: %s'%muteData['reason'])
	playerMuteInfoMenu.addoption('muter', 'Muter: %s'%muteData['muter'])
	playerMuteInfoMenu.addoption(['unmute', choice], 'Unmute')

def playerMuteInfoMenuSelect(userid, choice, popupid):
	if choice[0] == 'unmute':
		steamid = choice[1]
		if not steamid in mute.muted:
			es.tell(userid, '#multi', '#lightgreen[Mute]: #defaultPlayer is no longer muted!')
			return

		es.tell(userid, '#multi', '#lightgreen[Mute]: %s#default has been unmuted!'%mute.muted[steamid]['name'])
		mute.muted[steamid]['time'] = 2

def muteMenuSelect(userid, choice, popupid):
	chooseTimeMenu = popuplib.easymenu('Choose Time', '_popup_choice', chooseTimeMenuSelect)
	chooseTimeMenu.setdescription('In Minutes')

	for a in [5, 10, 15, 30, 60, 120, 180, 300, 600, 35791394]:
		chooseTimeMenu.addoption([choice, a * 60], a)
	popuplib.send('Choose Time', userid)

def chooseTimeMenuSelect(userid, choice, popupid):
	chooseReasonMenu = popuplib.easymenu('Choose Reason', '_popup_choice', chooseReasonMenuSelect)
	for reason in str(sv('bmute_menu_mute_reasons')).split('|'):
		chooseReasonMenu.addoption([choice[0], choice[1], reason], reason)
	popuplib.send('Choose Reason', userid)

def chooseReasonMenuSelect(userid, choice, popupid):
	mute.addPlayer(choice[0], choice[1], choice[2], es.getplayername(userid))
	es.tell(userid, '#multi', '#lightgreen[Mute]: %s#default has been muted!'%choice[0])
